--------------------------------------------------------------------------------------------------
-- script : ade-config.lua
-- author : Dan Gill
-- September 2018
-- version: 1.1
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
-- desc   : The intention of this script is to set the ade configs throughout the environment.  
-- slaves that are not present on master. (The ade probe automatically replicates packages from the
-- This script should run from the primary hub's nas. The folder "script_logs" *MUST* be manually
-- created under the nas directory prior to running this script.
---------------------------------------------------------------------------------------------------        
---------------------------------------------------------------------------------------------------
-- Version  | Details
---------------------------------------------------------------------------------------------------
-- 1.0      | Initial Version
-- 1.1      | 9/20/2018 - Replace dofile with require and added package.path update; this makes
--            passing the path to the includes files simpler. Swapped out file.write and print
--            statements with output function from logging_functions.
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------

package.path = package.path .. ";./scripts/includes/?.lua" -- Where are your require files?

local output_location = 2 -- 1 = stdout; 2 = file; 3 = both

local str_beg, str_end = string.find (SCRIPT_NAME,".",1,true)
local fname = "./script_logs/" .. left (SCRIPT_NAME, str_beg-1) .. ".log"


-----------------------------------------------------------------------
-----------------------------------------------------------------------
-- DO NOT EDIT BELOW THIS LINE
-----------------------------------------------------------------------
-----------------------------------------------------------------------

require ("error_functions")
require ("logging_functions")
require ("table_functions")

local probe = "automated_deployment_engine"
local robot_master = ""

if SCRIPT_ARGUMENT ~= nil then
   parms = split(SCRIPT_ARGUMENT)
   for k,v in ipairs (parms) do
      if k == 1 then
         robot_master = v
      end
   end
else
   local host = nimbus.request("controller", "get_info")
   robot_master = "/" .. host.domain .. "/" .. host.hubname .. "/" .. host.robotname
end

local probe_master = robot_master .. "/" .. probe

-----------------------------------------------------------------------
-----------------------------------------------------------------------
-- Define functions
-----------------------------------------------------------------------
-----------------------------------------------------------------------

-- Return a list of all hubs under the domain
local function get_hublist()

  local hubs = nimbus.request("hub", "gethubs")
  return hubs.hublist

end

-- Check if a probe is running on a host; returns 1 if installed and running and 0 if not.
local function check_probe_running(controller, probe)

   local args = pds.create()
   pds.putString(args, "name", probe)
   
   local probe_details, rc = nimbus.request(controller, "probe_list" , args)
   pds.delete(args)

   if rc == 0 then
      return probe_details[probe].active
   else
      return 0
   end
end

-- Return a table with the full path to specified probe if it is running; excludes "exclude" (send nil exclude to return all)
local function get_probe_paths(exclude, robotlist, probe)
   local probe_paths = {}
   i=1
   for index,_ in pairs(robotlist) do
      local hub = "/" .. robotlist[index].domain .. "/" .. robotlist[index].name .. "/" .. robotlist[index].robotname
      if hub ~= exclude then
         local active = check_probe_running("/" .. robotlist[index].domain .. "/" .. robotlist[index].name .. "/" .. robotlist[index].robotname .. "/controller", probe)
         if active == 1 then
            probe_paths[i] = "/" .. robotlist[index].domain .. "/" .. robotlist[index].name .. "/" .. robotlist[index].robotname .. "/" .. probe
            i = i+1
         end
      end
   end
   
   return probe_paths
end

local function set_sync_rules(master)

   local args = pds.create()
   pds.putString(args, "name", "*")
   pds.putString(args, "rule_type", "ALL")
   
   local resp, rc = nimbus.request(master, "add_package_sync_rule", args)
   
   pds.delete(args)
   
   if rc == 0 then
      output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Package Sync Rules added to master. Using * and ALL.", output_location)
--      file.write(fname,timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Package Sync Rules added to master. Using * and ALL.\n")
      return 0
   else
      output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Failed to configure package sync rules on master.", output_location)
--      file.write(fname,timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Failed to configure package sync rules on master.\n")
      return rc
   end

end

local function check_sync_master(master, slave)

   local ade_info,rc = nimbus.request(slave, "get_info")
   
   if rc == 0 then
      if ade_info.sync_master == master then
         output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Master is already set correctly on " .. slave, output_location)
--         file.write(fname,timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Master is already set correctly on " .. slave .. "\n")
         return 0
      else
         output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Master is incorrectly set on " .. slave .. " current master = " .. ade_info.sync_master, output_location)
--         file.write(fname,timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Master is incorrectly set on " .. slave .. " current master = " .. ade_info.sync_master .. "\n")
         return 1
      end
   else
      output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Failed to get info from " .. slave, output_location)
--      file.write(fname,timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Failed to get info from " .. slave .. "\n")
      return 0
   end
end

local function set_package_sync_master(master, ade_list)

   local skip_count = 0
   local change_count = 0

   for _,slave in ipairs (ade_list) do
      local update_master = check_sync_master(master.."/automated_deployment_engine", slave)
      if update_master == 0 then
         output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Skipping slave " .. slave .. ".\n", output_location)
--         file.write(fname,timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Skipping slave " .. slave .. ".\n\n")
         skip_count = skip_count + 1
      else
         local args = pds.create()
         pds.putString(args, "robot", master)
         local rc = nimbus.request(slave, "set_package_sync_master", args)
         pds.delete(args)
         output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Package sync master set to " .. master .. " on " .. slave .. ".\n", output_location)
--         file.write(fname,timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Package sync master set to " .. master .. " on " .. slave .. ".\n\n")
         change_count = change_count + 1
      end
   end
   
   return skip_count, change_count

end

local function main()
   -- Creates/overwrites the file - only do once
   if output_location == 2 or output_location == 3 then
      file.create (fname)
   end

   output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Setting robot_master to " .. robot_master, output_location)
--   file.write (fname,timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Setting robot_master to " .. robot_master .."\n")

   local hublist = get_hublist()
   
   local ade_paths = get_probe_paths(robot_master, hublist, probe)
   
   set_sync_rules(probe_master)
   
   local skips, changes = set_package_sync_master(robot_master, ade_paths)
   
   output(fname, timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Finished configuring ADE. Skipped " .. skips .. " ADEs and changed " .. changes .. " ADEs.", output_location)
--   file.write (fname,timestamp.format ( timestamp.now(), "%Y-%m-%d %H:%M:%S").." Finished configuring ADE. Skipped " .. skips .. " ADEs and changed " .. changes .. " ADEs.\n")
   
end

main()